﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Combinatorics
{
    class Program
    {
        static void Main(string[] args)
        {
            Permutations();
            Combinations();
            Console.ReadLine();
        }

        static int Factorial(int n)
        {
            if (n <= 1)
            {
                return 1;
            }
            
            return n * Factorial(n - 1);
        }

        static void Permutations()
        {
            int n = int.Parse(Console.ReadLine());
            int k = int.Parse(Console.ReadLine());

            int totalPermutations = Factorial(n) / Factorial(n - k);
            Console.WriteLine($"Total Permutations: {totalPermutations}");

            // Generate all elements from 1 to n in a list.
            var elementPool = Enumerable.Range(1, n);

            // We need somewhere to store our permutations.
            List<List<int>> permutations = new List<List<int>>();

            // Give our method a list of elements to pick from (elementPool).
            // Tell it how many to pick (k) and where to store the results (permutations).
            GeneratePermutations(elementPool, k, permutations);

            // Print each permutation
            foreach (var list in permutations)
            {
                Console.WriteLine(string.Join(", ", list));
            }
        }

        static void GeneratePermutations(IEnumerable<int> elements, int k, List<List<int>> store, IEnumerable<int> current = null)
        {
            if (current == null)
            {
                current = new List<int>();
            }

            if (current.Count() < k)
            {
                foreach (var element in elements.Except(current))
                {
                    var newCurrent = current.Append(element);
                    GeneratePermutations(elements, k, store, newCurrent);
                }
            }
            else
            {
                store.Add(current.ToList());
            }
        }

        static void Combinations()
        {
            int n = int.Parse(Console.ReadLine());
            int k = int.Parse(Console.ReadLine());

            int totalCombinations = Factorial(n) / (Factorial(k) * Factorial(n - k));
            Console.WriteLine($"Total Combinations: {totalCombinations}");

            // Generate all elements from 1 to n in a list.
            var elementPool = Enumerable.Range(1, n);

            // We need somewhere to store our permutations.
            List<List<int>> combinations = new List<List<int>>();

            // Give our method a list of elements to pick from (elementPool).
            // Tell it how many to pick (k) and where to store the results (permutations).
            GenerateCombinations(elementPool, k, combinations);

            // Print each combination
            foreach (var list in combinations)
            {
                Console.WriteLine(string.Join(", ", list));
            }
        }

        static void GenerateCombinations(IEnumerable<int> elements, int k, List<List<int>> store, IEnumerable<int> current = null)
        {
            if (current == null)
            {
                current = new List<int>();
            }

            if (current.Count() < k)
            {
                for (int i = 0; i < elements.Count(); i++)
                {
                    var element = elements.ElementAt(i);
                    var newCurrent = current.Append(element);
                    // The only difference here is that once an element has been used
                    // It cannot occur anymore, so we skip it for our future iterations.
                    GenerateCombinations(elements.Skip(i+1), k, store, newCurrent);
                }
            }
            else
            {
                store.Add(current.ToList());
            }
        }


    }
}
